% leave_one_out.m
% ERD

temp=findobj('tag','PopupMenu1');
value=get(temp, 'Value');           

num_of_clases=length(train_data);


disp('Classifying Samples using Leaving-One-Out...')

%=============================================================
% Maximum Likelihood
%=============================================================

if value == 1
   
temp=zeros(1,num_of_classes);
error=zeros(1,num_of_classes);

for j=1:num_of_classes
   e=0;
   
   for i=1:length(train_data{j})
      g=train_data{j};
      tr_means=means;
      tr_covs=covs;
      
      te_pixel=g(:,i);
      tr_g=cat(2,g(:,1:i-1),g(:,i+1:length(train_data{j})));
      tr_means{j}=mean(tr_g');
      tr_covs{j}=cov(tr_g');
      
      for k=1:num_of_classes
         temp(k)=maxlike(te_pixel,tr_means{k}',tr_covs{k});
      end
      
      if temp(j)==max(temp)
      e=e+1;
      end
      
   end
   error(j)=e;

end

etotal=0;
stotal=0;

disp('For Maximum Likelihood Classifier...')
disp(' ')

for i=1:num_of_classes
   disp(sprintf('The accuracy of the samples of Class %d is %.4f%%. \n', i, error(i)/length(train_data{i})*100))
   etotal=etotal + error(i);
   stotal=stotal + length(train_data{i});
end

disp(sprintf('The accuracy of the total samples is %.4f%%. \n', etotal/stotal*100))

%=============================================================
% Euclidean Distance
%=============================================================

elseif value == 2
   
temp=zeros(1,num_of_classes);
error=zeros(1,num_of_classes);

for j=1:num_of_classes
   e=0;
   
   for i=1:length(train_data{j})
      g=train_data{j};
      tr_means=means;
      tr_covs=covs;
      
      te_pixel=g(:,i);
      tr_g=cat(2,g(:,1:i-1),g(:,i+1:length(train_data{j})));
      tr_means{j}=mean(tr_g');
            
      for k=1:num_of_classes
         temp(k)=euclid(te_pixel,tr_means{k}');
      end
      
      if temp(j)==max(temp)
      e=e+1;
      end
      
   end
   error(j)=e;

end

etotal=0;
stotal=0;

disp('For Euclidean Distance Classifier...')
disp(' ')

for i=1:num_of_classes
   disp(sprintf('The accuracy of the samples of Class %d is %.4f%%. \n', i, error(i)/length(train_data{i})*100))
   etotal=etotal + error(i);
   stotal=stotal + length(train_data{i});
end

disp(sprintf('The accuracy of the total samples is %.4f%%. \n', etotal/stotal*100))

%=============================================================
% Fisher Linear Discriminant
%=============================================================

elseif value == 3 
   
temp=zeros(1,num_of_classes);
error=zeros(1,num_of_classes);


for j=1:num_of_classes
   e=0;
   
   for i=1:length(train_data{j})
      g=train_data{j};
      tr_means=means;
      tr_covs=covs;
      
      te_pixel=g(:,i);
      tr_g=cat(2,g(:,1:i-1),g(:,i+1:length(train_data{j})));
      tr_means{j}=mean(tr_g');
      tr_covs{j}=cov(tr_g');
      
      s_cov=0;

      for n=1:num_of_classes
         s_cov=s_cov + tr_covs{n};
      end

      a_cov=s_cov/num_of_classes;
      
      for k=1:num_of_classes
         temp(k)=fishdis(te_pixel,tr_means{k}',a_cov);
      end
      
      if temp(j)==max(temp)
      e=e+1;
      end
      
   end
   error(j)=e;

end

etotal=0;
stotal=0;

disp('For Fisher Linear Discriminant Classifier...')
disp(' ')

for i=1:num_of_classes
   disp(sprintf('The accuracy of the samples of Class %d is %.4f%%. \n', i, error(i)/length(train_data{i})*100))
   etotal=etotal + error(i);
   stotal=stotal + length(train_data{i});
end

disp(sprintf('The accuracy of the total samples is %.4f%%. \n', etotal/stotal*100))

%=============================================================
% Mahalanobis Distance
%=============================================================

elseif value == 4
   
temp=zeros(1,num_of_classes);
error=zeros(1,num_of_classes);

for j=1:num_of_classes
   e=0;
   
   for i=1:length(train_data{j})
      g=train_data{j};
      tr_means=means;
      tr_covs=covs;
      
      te_pixel=g(:,i);
      tr_g=cat(2,g(:,1:i-1),g(:,i+1:length(train_data{j})));
      tr_means{j}=mean(tr_g');
      tr_covs{j}=cov(tr_g');
      
      for k=1:num_of_classes
         temp(k)=maha(te_pixel,tr_means{k}',tr_covs{k});
      end
      
      if temp(j)==max(temp)
      e=e+1;
      end
      
   end
   error(j)=e;

end

etotal=0;
stotal=0;

disp('For Mahalanobis Distance Classifier...')
disp(' ')

for i=1:num_of_classes
   disp(sprintf('The accuracy of the samples of Class %d is %.4f%%. \n', i, error(i)/length(train_data{i})*100))
   etotal=etotal + error(i);
   stotal=stotal + length(train_data{i});
end

disp(sprintf('The accuracy of the total samples is %.4f%%. \n', etotal/stotal*100))

%=============================================================
% Angle Detection
%=============================================================

elseif value == 5
   
temp=zeros(1,num_of_classes);
error=zeros(1,num_of_classes);

for j=1:num_of_classes
   e=0;
   
   for i=1:length(train_data{j})
      g=train_data{j};
      tr_means=means;
      tr_covs=covs;
      
      te_pixel=g(:,i);
      tr_g=cat(2,g(:,1:i-1),g(:,i+1:length(train_data{j})));
      tr_means{j}=mean(tr_g');
            
      for k=1:num_of_classes
         temp(k)=angledet(te_pixel,tr_means{k}');
      end
      
      if temp(j)==min(temp)
      e=e+1;
      end
      
   end
   error(j)=e;

end

etotal=0;
stotal=0;

disp('For Angle Detection ...')
disp(' ')

for i=1:num_of_classes
   disp(sprintf('The accuracy of the samples of Class %d is %.4f%%. \n', i, error(i)/length(train_data{i})*100))
   etotal=etotal + error(i);
   stotal=stotal + length(train_data{i});
end

disp(sprintf('The accuracy of the total samples is %.4f%%. \n', etotal/stotal*100))

end

clear i j k n etotal e stotal a_cov g s_cov temp error maxi fish euc mah 
clear tag_maxi tag_fish tag_euc tag_mah tr_covs tr_means te_pixel tr_g
